import {IConnectionInfo} from "./IConnectionInfo";
import {ConnectionInfo} from "./ConnectionInfo";
import {IQueryResult} from "./IQueryResult";

/**
 * Created by mhainc on 2/22/2017.
 */
declare function require(name:string);
var module = require('./module.js');

export class Client {
    private internalClient : any;
    constructor(host:string, port:number, username:string, password:string, account:string, cacertPath:string = null) {
        this.internalClient = module(host, port, username, password, account, cacertPath);
    }

    private ConnectInternal(onConnectedCallback:(result: IConnectionInfo)=>any, onErrorCallback:(error: string)=>any) {
        this.internalClient.connect(onConnectedCallback, onErrorCallback);
    }

    public Connect(): Promise<IConnectionInfo> {
        var result = new Promise<IConnectionInfo>((resolve, reject) =>{
            this.ConnectInternal(resolve, reject);
        });        
        return result;
    }

    private ExecuteInternal<T>(sql:string, params:Array<any>, onCompletedCallback:(result: Array<T>)=>any, onErrorCallback:(error:string)=>any)
    {
        if (sql == null)
        {
            throw new Error("sql cannot be null");
        }
        this.internalClient.execute(sql, params, onCompletedCallback, onErrorCallback);
    }

    public Execute<T>(sql:string, params:Array<any>): Promise<Array<T>> {
        var result = new Promise<Array<T>>((resolve, reject)=>{
            this.ExecuteInternal(sql, params, resolve, reject);
        });
        return result;
    }

    private DisconnectInternal(onDisconnectedCallback:()=>any, onErrorCallback:(error:string)=>any=null) {
        this.internalClient.disconnect(onDisconnectedCallback, onErrorCallback);
    }

    public Disconnect(): Promise<boolean> {
        var result = new Promise<boolean>((resolve, reject) => {
            this.DisconnectInternal(resolve, reject);
        });
        return result;
    }
}
