/**
 * Connect Bridge REST WS wrapper
 */
var https = require('https');
var fs = require('fs');
var jsStringEscape = require('js-string-escape');

module.exports = function (host, port, username, password, account, cacertPath) {
    var self = this;
    self.cacert = null;
    if (cacertPath) {
        self.cacert = fs.readFileSync(cacertPath);
    };
    self.host = host;
    self.port = port;
    self.username = username;
    self.password = password;
    self.account = account;
    self.connectionRaw = '';
    self.connection = null;
    self.lastResultRaw = '';
    
    //connects to the connect bridge web service
    //and stores the connection internally
    self.connect = function (complete, error) {
        var wraper = {
            'username': self.username,
            'password': self.password,
            'accountname': self.account
        };
        var op = "/WsDriver/rest/CreateConnection?format=json";
        var postData = JSON.stringify(wraper);
        
        var req = https.request({
            hostname: self.host,
            port: self.port,
            path: op,
            ca: self.cacert,
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Content-Length': postData.length
            }
        }, function (res) {            
            self.connectionRaw = '';
            res.setEncoding('utf8');
            res.on('data', function (chunk) {
                self.connectionRaw += chunk;
            });
            res.on('end', function () {
                self.connection = JSON.parse(self.connectionRaw);
                if (self.connection.IsSuccess === true) {
                    if (complete) complete(self.connection);
                }
                else { 
                    if (error) error(self.connection.ErrorMessage);
                }
                self.connectionRaw = '';
            });
        });
        
        req.write(postData);
        req.end();
    };
    
    //executes a sql statement against connect bridge
    //with parameters
    self.execute = function (sql, params, complete, error) {
        var op = "/WsDriver/rest/Execute?format=json";
        var stmt = {
            'SqlStatement': sql,
            'Parameters': params
        };
        var wraper = {
            'connection': self.connection,
            'command': stmt
        };
        
        var postData = JSON.stringify(wraper);

        var req = https.request({
            hostname: self.host,
            port: self.port,
            ca: self.cacert,
            path: op,
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Content-Length': postData.length
            }
        }, function (res) {
            self.connectionRaw = '';
            res.setEncoding('utf8');
            res.on('data', function (chunk) {
                self.lastResultRaw += chunk;
            });
            res.on('end', function () {
                var result = JSON.parse(self.lastResultRaw);
                if (result.IsSuccess === true) {
                    if (complete) self.beautifyResult(result, complete);
                }
                else {
                    if (error) error(result.ErrorMessage);
                }
                self.lastResultRaw = '';
            });
        });
        
        req.write(postData);
        req.end();
    };

    //closes the connection
    self.disconnect = function (complete, error) {
        var op = "/WsDriver/rest/CloseConnection?format=json";
        var postData = JSON.stringify(self.connection);
        
        var req = https.request({
            hostname: self.host,
            port: self.port,
            path: op,
            ca: self.cacert,
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Content-Length': postData.length
            }
        }, function (res) {
            self.connectionRaw = '';
            res.setEncoding('utf8');
            res.on('data', function (chunk) {
                self.lastResultRaw += chunk;
            });
            res.on('end', function () {
                //var result = JSON.parse(self.lastResultRaw);
                //if (result.IsSuccess === true) {
                if (complete) complete();
                //}
                //else {
                //    if (error) error(result.ErrorMessage);
                //}
                self.lastResultRaw = '';
            });
        });
        
        req.write(postData);
        req.end();        
    };
       
    self.beautifyResult = function (result, complete) {
        var r = result.Row.map(function (item) {
            var json = "{";
            for (var i = 0; i < result.ColumnMetaData.length; i++) {
                var colInfo = result.ColumnMetaData[i];
                json += "\"" + colInfo.ItemName + "\":";
                var val = item[i];
                if (val === null) {
                    json += "null";
                }
                else {
                    if (colInfo.Type === 13) {
                        if (typeof (val) !== "undefined" && val !== null) {
                            //json += "\"";
                            json += self.escapedStringify(item[i],true);
                            //json += "\"";
                        }
                        else {
                            json += "null";
                        }
                    }
                    else {
                        /*
                        case0:
                        return typeof(sbyte);
                        case 1:
                        return typeof(byte);
                        case 2:
                        return typeof(Int16);
                        case 3:
                        return typeof(UInt16);
                        case 4:
                        return typeof(Int32);
                        case 5:
                        return typeof(UInt32);
                        case 6:
                        return typeof(Int64);
                        case 7:
                        return typeof(UInt64);
                        case 8:
                        return typeof(Single);
                        case 9:
                        return typeof(Double);
                        case 10:
                        return typeof(Decimal);
                        case 11:
                        return typeof(char);
                        case 12:
                        return typeof(bool);
                        case 13:
                        return typeof(string);
                        case 14:
                        return typeof(DateTime);
                        case 15: 
                        return typeof(byte[]);
                        case 16:
                        return typeof(string);
                        case 17: 
                        return typeof(Int64);
                        default:
                        return typeof(string);
                        */

                        if (colInfo.Type === 14) {
                            val = "\"" + self.toDate(item[i]).toJSON() + "\"";
                        }
                        else if (colInfo.Type === 15) {
                            val = "[" + val + "]";
                        }
                        if (typeof (val) === "undefined" || val === null) {
                            val = "null";
                        }
                        json += val;
                    }
                }
                json += ",";
            }
            json = json.substring(0, json.length - 1);
            json += "}";
                
            //console.log(json);
            return JSON.parse(json);
        });
        if (complete) complete(r);
    };
    
    self.escapedStringify = function (s, emit_unicode) {
        var json = JSON.stringify(s);
        return emit_unicode ? json : json.replace(/\//g,
      function (c) {
            return '\\/';
        }
        ).replace(/[\u003c\u003e]/g,
      function (c) {
            return '\\u' + ('0000' + c.charCodeAt(0).toString(16)).slice(-4).toUpperCase();
        }
        ).replace(/[\u007f-\uffff]/g,
      function (c) {
            return '\\u' + ('0000' + c.charCodeAt(0).toString(16)).slice(-4);
        }
        );
    };

    self.escapeJsonStringVal = function (strVal) {
        /*
        return strVal       
        .replace(/\\n/g, "\\n")
        .replace(/\\'/g, "\\'")
        .replace(/\\"/g, '\\"')
        .replace(/\\&/g, "\\&")
        .replace(/\\r/g, "\\r")
        .replace(/\\t/g, "\\t")
        .replace(/\\b/g, "\\b")
        .replace(/\\f/g, "\\f")
        .replace(/\\f/g, "\\f")
        .replace("\\", "\\\\");
         */
     };
        
    self.toDate = function (mgDt) {
        //TODO: process local and utc times correctly using Hour offset provided in mgDt
        return new Date(parseInt(mgDt.Year), parseInt(mgDt.Month), parseInt(mgDt.Day),
                parseInt(mgDt.Hour), parseInt(mgDt.Minute), parseInt(mgDt.Second),
                parseInt(mgDt.Fraction) * 100);
    };
    
    return self;
};
